import numpy as np

def normalize(feature_in_rows, method, blocksize):
  options = {'l2norm': l2norm, 'power_l2norm': power_l2norm, 'power_intra_l2norm': power_intra_l2norm}
  feature = options[method](feature_in_rows, blocksize)
  return feature

# power,intra-norm, L2
def power_intra_l2norm(feature_in_rows, blocksize):
  # power-normalization
  nr = feature_in_rows.shape[0]
  dim = feature_in_rows.shape[1]
  feature_in_rows = np.sign(feature_in_rows) * np.abs(feature_in_rows) ** 0.5
  #intra-norm
  tmp_tr = np.reshape(feature_in_rows, (-1,blocksize), order='C')
  norms = np.sqrt(np.sum(tmp_tr ** 2, 1)) + np.finfo(float).eps
  tmp_tr /= norms.reshape(-1, 1)
  feature_in_rows = np.reshape(tmp_tr,(nr,dim))
  # L2 normalize
  norms = np.sqrt(np.sum(feature_in_rows ** 2, 1)) + np.finfo(float).eps
  feature_in_rows /= norms.reshape(-1, 1)
  return feature_in_rows
  
# power and L2
def power_l2norm(feature_in_rows, blocksize=None):
  nr = feature_in_rows.shape[0]
  dim = feature_in_rows.shape[1]
  feature_in_rows = np.sign(feature_in_rows) * np.abs(feature_in_rows) ** 0.5
  norms = np.sqrt(np.sum(feature_in_rows ** 2, 1)) + np.finfo(float).eps
  feature_in_rows /= norms.reshape(-1, 1)
  return feature_in_rows
  
# L2
def l2norm(feature_in_rows, blocksize=None):
  nr = feature_in_rows.shape[0]
  dim = feature_in_rows.shape[1]
  norms = np.sqrt(np.sum(feature_in_rows ** 2, 1)) + np.finfo(float).eps
  feature_in_rows /= norms.reshape(-1, 1)
  return feature_in_rows
  
